@@ -16,6 +16,6 @@ public class FormatSource {
 		MasonTagTypes.register();
 		Source source=new Source(new URL(sourceUrlString));
 
-		new SourceFormatter(source).setIndentString("  ").setTidyTags(true).writeTo(new OutputStreamWriter(System.out));
+		new SourceFormatter(source).setIndentString("\t").setTidyTags(true).setCollapseWhiteSpace(true).writeTo(new OutputStreamWriter(System.out));
 	}
 }
