@@ -203,10 +203,23 @@ public class HTMLSanitiser {
 		List<Tag> tags=source.getAllTags();
 		int pos=0;
 	  for (Tag tag : tags) {
+	  	if (tag.getBegin()<pos) continue; // this might happen if a script element was encountered
 			if (processTag(tag,outputDocument)) {
 			  tag.setUserData(VALID_MARKER);
 			} else {
 				if (!stripInvalidElements) continue; // element will be encoded along with surrounding text
+				if (tag.getName()==HTMLElementName.SCRIPT && tag.getTagType()==StartTagType.NORMAL) {
+					EndTag endTag=tag.getElement().getEndTag();
+					if (endTag==null) {
+						// script has no end tag, remove everything to the end of the source and don't process any more tags
+						outputDocument.remove(tag.getBegin(),source.getEnd());
+						return outputDocument.toString();
+					}
+					// remove the whole script element including tags and content
+					outputDocument.remove(tag.getBegin(),endTag.getEnd());
+					pos=endTag.getEnd();
+					continue;
+				}
 				outputDocument.remove(tag);
 			}
 			reencodeTextSegment(source,outputDocument,pos,tag.getBegin(),formatWhiteSpace);
@@ -300,7 +313,7 @@ public class HTMLSanitiser {
 		displayStripInvalidMarkup("ab & c","encode text");
 		displayStripInvalidMarkup("abc <u>def</u> geh","<U> element not allowed");
 		displayStripInvalidMarkup("<p>abc","add optional end tag");
-		displayStripInvalidMarkup("<script>abc</script>","remove potentially dangerous script");
+		displayStripInvalidMarkup("abc<script>def</script>geh","remove potentially dangerous script");
 		displayStripInvalidMarkup("<p class=\"xyz\" onmouseover=\"nastyscript\">abc</p>","keep approved attributes but strip non-approved attributes");
 		displayStripInvalidMarkup("<p id=abc class='xyz'>abc</p>","tidy up attributes to make them XHTML compliant");
 		displayStripInvalidMarkup("List:<ul><li>A</li><li>B<li>C</ul>","inserts optional end tags");
