@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -50,8 +54,8 @@ import java.io.*;
  * @see StartTag
  * @see Attribute
  */
-public final class Attributes extends SequentialListSegment<Attribute> {
-	private final LinkedList<Attribute> attributeList; // never null
+public final class Attributes extends ListSegment<Attribute> {
+	private final ArrayList<Attribute> attributeList; // never null
 	
 	final boolean containsServerTagOutsideOfAttributeValue;
 
@@ -67,7 +71,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 
 	private static int defaultMaxErrorCount=2; // defines maximum number of minor errors that can be encountered in attributes before entire start tag is rejected.
 
-	private Attributes(final Source source, final int begin, final int end, final LinkedList<Attribute> attributeList, final boolean containsServerTagOutsideOfAttributeValue) {
+	private Attributes(final Source source, final int begin, final int end, final ArrayList<Attribute> attributeList, final boolean containsServerTagOutsideOfAttributeValue) {
 		super(source,begin,end);
 		this.attributeList=attributeList;
 		this.containsServerTagOutsideOfAttributeValue=containsServerTagOutsideOfAttributeValue;
@@ -106,7 +110,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 			attributesBegin=logBegin;
 		}
 		int attributesEnd=attributesBegin;
-		final LinkedList<Attribute> attributeList=new LinkedList<Attribute>();
+		final ArrayList<Attribute> attributeList=new ArrayList<Attribute>();
 		boolean containsServerTagOutsideOfAttributeValue=false;
 		final ParseText parseText=source.getParseText();
 		int i=attributesBegin;
@@ -146,7 +150,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 							} else {
 								if (isTerminatingCharacter) {
 									if (i==maxEnd) {
-										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"terminated in the middle of a quoted attribute value",i);
+										if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"terminated in the middle of a quoted attribute value",i);
 										if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 										valueSegment=new Segment(source,currentBegin,i);
 										valueSegmentIncludingQuotes=new Segment(source,currentBegin-1,i); // this is missing the end quote
@@ -164,7 +168,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 							attributesEnd=valueSegmentIncludingQuotes.getEnd();
 							parsingState=ParsingState.BETWEEN_ATTRIBUTES;
 						} else if (ch=='<' && quote==' ') {
-							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in unquoted attribute value",i);
+							if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in unquoted attribute value",i);
 							return null;
 						}
 						break;
@@ -181,11 +185,11 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 						} else if (!Tag.isXMLNameChar(ch)) {
 							// invalid character detected in attribute name.
 							if (ch=='<') {
-								if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in attribute name",i);
+								if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character in attribute name",i);
 								return null;
 							}
 							if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
-							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid character",i);
+							if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid character",i);
 							if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 						}
 						break;
@@ -201,7 +205,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 						} else if (ch=='=') {
 							parsingState=ParsingState.START_VALUE;
 						} else if (ch=='<') {
-							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character after attribute name",i);
+							if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character after attribute name",i);
 							return null;
 						}
 						break;
@@ -212,23 +216,23 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 								quote=' ';
 							} else {
 								if (quote!=' ') {
-									if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"has missing whitespace after quoted attribute value",i);
+									if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"has missing whitespace after quoted attribute value",i);
 									// only count this as an error if there have already been other errors, otherwise allow unlimited errors of this type.
 									if (errorCount>0 && reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 								}
 								if (!Tag.isXMLNameStartChar(ch)) {
 									// invalid character detected as first character of attribute name.
 									if (ch=='<') {
-										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character",i);
+										if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character",i);
 										return null;
 									}
 									if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
 									if (startTagType==StartTagType.NORMAL && startTagType.atEndOfAttributes(source,i,false)) {
 										// This checks whether we've found the characters "/>" but it wasn't recognised as the closing delimiter because isClosingSlashIgnored is true.
-										if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
+										if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
 										break;
 									}
-									if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid first character",i);
+									if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"contains attribute name with invalid first character",i);
 									if (reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 								}
 								parsingState=ParsingState.IN_NAME;
@@ -239,7 +243,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 					case START_VALUE:
 						currentBegin=i;
 						if (isTerminatingCharacter) {
-							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"has missing attribute value after '=' sign",i);
+							if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"has missing attribute value after '=' sign",i);
 							// only count this as an error if there have already been other errors, otherwise allow unlimited errors of this type.
 							if (errorCount>0 && reachedMaxErrorCount(++errorCount,source,logType,tagName,logBegin,maxErrorCount)) return null;
 							final Segment valueSegment=new Segment(source,i,i);
@@ -254,7 +258,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 						} else if (isWhiteSpace(ch)) {
 							break; // just ignore whitespace after the '=' sign as nearly all browsers do.
 						} else if (ch=='<') {
-							if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character at the start of an attribute value",i);
+							if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because of '<' character at the start of an attribute value",i);
 							return null;
 						} else {
 							quote=' ';
@@ -265,7 +269,7 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 						if (!isTerminatingCharacter) {
 							if (!isWhiteSpace(ch)) {
 								if (isInvalidEmptyElementTag(startTagType,source,i,logType,tagName,logBegin)) break;
-								if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because the name contains an invalid character",i);
+								if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because the name contains an invalid character",i);
 								return null;
 							}
 							parsingState=ParsingState.BETWEEN_ATTRIBUTES;
@@ -276,24 +280,33 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 			}
 			return new Attributes(source,attributesBegin,attributesEnd,attributeList,containsServerTagOutsideOfAttributeValue);
 		} catch (IndexOutOfBoundsException ex) {
-			if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because it has no closing '>' character");
+			if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because it has no closing '>' character");
 			return null;
 		}
 	}
 
 	private static boolean reachedMaxErrorCount(final int errorCount, final Source source, final String logType, final String tagName, final int logBegin, final int maxErrorCount) {
 		if (errorCount<=maxErrorCount) return false;
-		if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"rejected because it contains too many errors");
+		if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"rejected because it contains too many errors");
 		return true;
 	}
 
 	private static boolean isInvalidEmptyElementTag(final StartTagType startTagType, final Source source, final int i, final String logType, final String tagName, final int logBegin) {
 		// This checks whether we've found the characters "/>" but it wasn't recognised as the closing delimiter because isClosingSlashIgnored is true.
 		if (startTagType!=StartTagType.NORMAL || !startTagType.atEndOfAttributes(source,i,false)) return false;
-		if (source.logger.isInfoEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
+		if (source.logger.isErrorEnabled()) log(source,logType,tagName,logBegin,"contains a '/' character before the closing '>', which is ignored because tags of this name cannot be empty-element tags");
 		return true;
 	}
 
+	/**
+	 * Returns the {@link Attribute} at the specified index.
+	 * @param index  the index of the attribute to get.
+	 * @return the attribute with the specified index.
+	 */
+	public Attribute get(final int index) {
+		return attributeList.get(index);
+	}
+
 	/**
 	 * Returns the {@link Attribute} with the specified name (case insensitive).
 	 * <p>
@@ -305,9 +318,9 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 	 * @see #getValue(String name)
 	 */
 	public Attribute get(final String name) {
-		if (size()==0) return null;
-		for (int i=0; i<size(); i++) {
-			final Attribute attribute=get(i);
+		final int size=attributeList.size();
+		for (int i=0; i<size; i++) {
+			final Attribute attribute=attributeList.get(i);
 			if (attribute.getKey().equalsIgnoreCase(name)) return attribute;
 		}
 		return null;
@@ -525,14 +538,15 @@ public final class Attributes extends SequentialListSegment<Attribute> {
 	
 	void setStartTag(final StartTag startTag) {
 		// this just preloads the startTag cache in each Attribute so we don't have to go looking for it if it is requested.
-		for (Attribute attribute : attributeList) attribute.startTag=startTag;
+		final int size=attributeList.size();
+		for (int i=0; i<size; i++) attributeList.get(i).startTag=startTag;
 	}
 
 	private static void log(final Source source, final String part1, final CharSequence part2, final int begin, final String part3, final int pos) {
-		source.logger.info(source.getRowColumnVector(pos).appendTo(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).append(" at position ")).toString());
+		source.logger.error(source.getRowColumnVector(pos).appendTo(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).append(" at position ")).toString());
 	}
 
 	private static void log(final Source source, final String part1, final CharSequence part2, final int begin, final String part3) {
-		source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).toString());
+		source.logger.error(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append(part1).append(' ').append(part2).append(" at ")).append(' ').append(part3).toString());
 	}
 }
