@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -143,51 +147,65 @@ public abstract class CharacterReference extends Segment {
 	}
 
 	/**
-	 * Encodes the specified text, escaping special characters into character references.
+	 * Encodes the specified text, escaping certain characters into character references.
 	 * <p>
-	 * Each character is encoded only if the {@link #requiresEncoding(char)} method would return <code>true</code> for that character,
-	 * using its {@link CharacterEntityReference} if available, or a decimal {@link NumericCharacterReference} if its unicode
-	 * code point is greater than U+007F.
+	 * This is equivalent to {@link #encode(CharSequence,boolean) encode(unencodedText,true)}.
+	 *
+	 * @param unencodedText  the text to encode.
+	 * @return the encoded string.
+	 */
+	public static String encode(final CharSequence unencodedText) {
+		return encode(unencodedText,true);
+	}
+
+	/**
+	 * Encodes the specified text, escaping certain characters into character references.
+	 * <p>
+	 * The {@link Config#CurrentCharacterReferenceEncodingBehaviour} setting determines which characters are encoded.
+	 * <p>
+	 * For characters that are to be encoded, the {@link CharacterEntityReference} is used if possible, otherwise a {@link NumericCharacterReference} is used.
 	 * <p>
 	 * The only exception to this is an {@linkplain CharacterEntityReference#_apos apostrophe} (U+0027),
-	 * which depending on the current setting of the static {@link Config#IsApostropheEncoded} property,
-	 * is either left unencoded (default setting), or encoded as the numeric character reference "<code>&amp;#39;</code>".
+	 * which is encoded as the numeric character reference "<code>&amp;#39;</code>" rather than its character entity reference {@link CharacterEntityReference#_apos &amp;apos;}
+	 * as this entity is not defined for use in HTML. See the comments in the {@link CharacterEntityReference} class for more information.
 	 * <p>
-	 * This method never encodes an apostrophe into its character entity reference {@link CharacterEntityReference#_apos &amp;apos;}
-	 * as this entity is not defined for use in HTML.  See the comments in the {@link CharacterEntityReference} class for more information.
+	 * Specifying a value of <code>true</code> as an argument to the <code>insideAttributeValue</code> parameter ensures that
+	 * double quote characters (<code>"</code>) are encoded. The default behaviour is that they are not encoded if a value of <code>false</code> is specified.
 	 * <p>
 	 * To encode text using only numeric character references, use the<br />
 	 * {@link NumericCharacterReference#encode(CharSequence)} method instead.
 	 *
 	 * @param unencodedText  the text to encode.
+	 * @param insideAttributeValue  specifies whether the output must be valid inside a quoted attribute value.
 	 * @return the encoded string.
 	 * @see #decode(CharSequence)
 	 */
-	public static String encode(final CharSequence unencodedText) {
+	public static String encode(final CharSequence unencodedText, final boolean insideAttributeValue) {
 		if (unencodedText==null) return null;
 		try {
-			return appendEncode(new StringBuilder(unencodedText.length()*2),unencodedText,false).toString();
+			return appendEncode(new StringBuilder(unencodedText.length()*2),unencodedText,insideAttributeValue).toString();
 		} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
 	}
 
 	/**
-	 * Encodes the specified character into a character reference if {@linkplain #requiresEncoding(char) required}.
+	 * Encodes the specified character into a character reference if {@linkplain Config#CurrentCharacterReferenceEncodingBehaviour required}.
 	 * <p>
-	 * The encoding of the character follows the same rules as for each character in the {@link #encode(CharSequence unencodedText)} method.
+	 * The encoding of the character follows the same rules as for each character in the {@link #encode(CharSequence unencodedText, boolean insideAttributeValue)} method,
+	 * with <code>insideAttributeValue</code> set to <code>true</code>.
 	 *
 	 * @param ch  the character to encode.
 	 * @return a character reference if appropriate, otherwise a string containing the original character.
 	 */
 	public static String encode(final char ch) {
 		try {
-			return appendEncode(new StringBuilder(MAX_ENTITY_REFERENCE_LENGTH),ch).toString();
+			return appendEncode(new StringBuilder(MAX_ENTITY_REFERENCE_LENGTH),ch,true).toString();
 		} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
 	}
 
 	/**
 	 * {@linkplain #encode(CharSequence) Encodes} the specified text, preserving line breaks, tabs and spaces for rendering by converting them to markup.
 	 * <p>
-	 * This performs the same encoding as the {@link #encode(CharSequence)} method, but also performs the following conversions:
+	 * This performs the same encoding as {@link #encode(CharSequence,boolean) encode(CharSequence,false)}, but also performs the following conversions:
 	 * <ul>
 	 *  <li>Line breaks, being Carriage Return (U+000D) or Line Feed (U+000A) characters, and Form Feed characters (U+000C)
 	 *   are converted to "<code>&lt;br /&gt;</code>".  CR/LF pairs are treated as a single line break.
@@ -209,10 +227,66 @@ public abstract class CharacterReference extends Segment {
 	 * @return the encoded string with white space formatting converted to markup.
 	 * @see #encode(CharSequence)
 	 */
-	public static String encodeWithWhiteSpaceFormatting(final CharSequence unencodedText) {
+	public static String encodeWithWhiteSpaceFormatting(CharSequence unencodedText) {
 		if (unencodedText==null) return null;
 		try {
-			return appendEncode(new StringBuilder(unencodedText.length()*2),unencodedText,true).toString();
+			Appendable appendable=new StringBuilder(unencodedText.length()*2);
+			int beginPos=0;
+			int endPos=unencodedText.length();
+			if (unencodedText instanceof Segment) {
+				// this might improve performance slightly
+				final Segment segment=(Segment)unencodedText;
+				final int segmentOffset=segment.getBegin();
+				beginPos=segmentOffset;
+				endPos+=segmentOffset;
+				unencodedText=segment.source;
+			}
+			for (int i=beginPos; i<endPos; i++) {
+				char ch=unencodedText.charAt(i);
+				if (!isWhiteSpace(ch)) {
+					appendEncode(appendable,ch,false);
+					continue;
+				}
+				// need to process white space
+				// whiteSpaceFormatting tries to simulate the formatting characters by converting them to markup
+				int spaceCount;
+				int nexti=i+1;
+				if (ch!=' ') {
+					if (ch!='\t') {
+						// must be line feed, carriage return or form feed, since zero-width space should have been processed as a character reference string
+						if (ch=='\r' && nexti<endPos && unencodedText.charAt(nexti)=='\n') i++; // process cr/lf pair as one line break
+						appendable.append("<br />"); // add line break
+						continue;
+					} else {
+						spaceCount=TAB_LENGTH;
+					}
+				} else {
+					spaceCount=1;
+				}
+				while (nexti<endPos) {
+					ch=unencodedText.charAt(nexti);
+					if (ch==' ')
+						spaceCount+=1;
+					else if (ch=='\t')
+						spaceCount+=TAB_LENGTH;
+					else
+						break;
+					nexti++;
+				}
+				if (spaceCount==1) {
+					// handle the very common case of a single character to improve efficiency slightly
+					appendable.append(' ');
+					continue;
+				}
+				if (spaceCount%2==1) appendable.append(' '); // fist character is a space if we have an odd number of spaces
+				while (spaceCount>=2) {
+					appendable.append("&nbsp; "); // use alternating &nbsp; and spaces to keep original number of spaces
+					spaceCount-=2;
+				}
+				// note that the last character is never a nbsp, so that word wrapping won't result in a nbsp before the first character in a line
+				i=nexti-1; // minus 1 because top level for loop will add it again
+			}
+			return appendable.toString();
 		} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
 	}
 
@@ -269,16 +343,14 @@ public abstract class CharacterReference extends Segment {
 		return decode(encodedText,insideAttributeValue,Config.ConvertNonBreakingSpaces);
 	}
 
-	static String decode(final CharSequence encodedText, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) {
+	static final String decode(final CharSequence encodedText, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) {
 		if (encodedText==null) return null;
-		for (int i=0; i<encodedText.length(); i++) {
-			if (encodedText.charAt(i)=='&') {
-				try {
-					return appendDecode(new StringBuilder(encodedText.length()),encodedText,i,insideAttributeValue,convertNonBreakingSpaces).toString();
-				} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
-			}
-		}
-		return encodedText.toString();
+		final String encodedTextString=encodedText.toString(); // converting to string first is faster than searching the CharSequence directly.
+		final int firstAmpersandPos=encodedTextString.indexOf('&');
+		if (firstAmpersandPos==-1) return encodedTextString;
+		try {
+			return appendDecode(new StringBuilder(encodedText.length()),encodedText,firstAmpersandPos,insideAttributeValue,convertNonBreakingSpaces).toString();
+		} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
 	}
 
 	/**
@@ -292,6 +364,8 @@ public abstract class CharacterReference extends Segment {
 	 * <p>
 	 * If the static {@link Config#ConvertNonBreakingSpaces} property is set to <code>true</code> (the default),
 	 * then all non-breaking space ({@link CharacterEntityReference#_nbsp &amp;nbsp;}) character entity references are converted to normal spaces.
+	 * For consistency with the rendered output of most user agents these converted spaces are not treated as white space,
+	 * so they are not collapsed and not trimmed.
 	 * <p>
 	 * <a href="#Unterminated">Unterminated</a> character references are dealt with according to the rules for
 	 * text outside of attribute values in the {@linkplain Config#CurrentCompatibilityMode current compatibility mode}.
@@ -315,15 +389,14 @@ public abstract class CharacterReference extends Segment {
 	 * <p>
 	 * This process ensures that the specified encoded text does not contain any remaining unencoded characters.
 	 * <p>
-	 * IMPLEMENTATION NOTE: At present this method simply calls the {@link #decode(CharSequence) decode} method
-	 * followed by the {@link #encode(CharSequence) encode} method, but a more efficient implementation
-	 * may be used in future.
+	 * IMPLEMENTATION NOTE: At present this method simply calls the {@link #decode(CharSequence) decode} method followed by the
+	 * {@link #encode(CharSequence) encode} method, both with <code>insideAttributeValue</code> set to <code>true</code>.
 	 *
 	 * @param encodedText  the text to re-encode.
 	 * @return the re-encoded string.
 	 */
 	public static String reencode(final CharSequence encodedText) {
-		return encode(decode(encodedText,true));
+		return encode(decode(encodedText,true),true);
 	}
 
 	/**
@@ -540,19 +613,12 @@ public abstract class CharacterReference extends Segment {
 	}
 
 	/**
-	 * Indicates whether the specified character would need to be encoded in HTML text.
-	 * <p>
-	 * This is the case if a {@linkplain CharacterEntityReference character entity reference} exists for the character, or the unicode code point is greater than U+007F.
-	 * <p>
-	 * The only exception to this is an {@linkplain CharacterEntityReference#_apos apostrophe} (U+0027),
-	 * which only returns <code>true</code> if the static {@link Config#IsApostropheEncoded} property
-	 * is currently set to <code>true</code>.
-	 *
-	 * @param ch  the character to test.
-	 * @return <code>true</code> if the specified character would need to be encoded in HTML text, otherwise <code>false</code>.
+	 * Indicates whether the specified character would need to be encoded in an attribute value.
+	 * @deprecated  use {@link Config#CurrentCharacterReferenceEncodingBehaviour} instead.
 	 */
+	@Deprecated
 	public static final boolean requiresEncoding(final char ch) {
-		return ch>127 || (CharacterEntityReference.getName(ch)!=null && (ch!='\'' || Config.IsApostropheEncoded));
+		return Config.CurrentCharacterReferenceEncodingBehaviour.isEncoded(ch,true);
 	}
 
 	/**
@@ -573,7 +639,7 @@ public abstract class CharacterReference extends Segment {
 		}
 		public void write(final char ch) throws IOException {
 			sb.setLength(0);
-			appendEncode(sb,ch);
+			appendEncode(sb,ch,true);
 			if (sb.length()==1)
 				out.write(sb.charAt(0));
 			else
@@ -592,12 +658,7 @@ public abstract class CharacterReference extends Segment {
 		}
 	}
 
-	private static Appendable appendEncode(final Appendable appendable, char ch) throws IOException {
-		if (appendEncodeCheckForWhiteSpaceFormatting(appendable,ch,false)) return appendable;
-		return appendable.append(ch);
-	}
-
-	static Appendable appendEncode(final Appendable appendable, CharSequence unencodedText, final boolean whiteSpaceFormatting) throws IOException {
+	static Appendable appendEncode(final Appendable appendable, CharSequence unencodedText, final boolean insideAttributeValue) throws IOException {
 		if (unencodedText==null) return appendable;
 		int beginPos=0;
 		int endPos=unencodedText.length();
@@ -609,71 +670,22 @@ public abstract class CharacterReference extends Segment {
 			endPos+=segmentOffset;
 			unencodedText=segment.source;
 		}
-		final boolean isApostropheEncoded=Config.IsApostropheEncoded;
-		for (int i=beginPos; i<endPos; i++) {
-			char ch=unencodedText.charAt(i);
-			if (appendEncodeCheckForWhiteSpaceFormatting(appendable,ch,whiteSpaceFormatting)) continue;
-			// need to process white space
-			// whiteSpaceFormatting tries to simulate the formatting characters by converting them to markup
-			int spaceCount;
-			int nexti=i+1;
-			if (ch!=' ') {
-				if (ch!='\t') {
-					// must be line feed, carriage return or form feed, since zero-width space should have been processed as a character reference string
-					if (ch=='\r' && nexti<endPos && unencodedText.charAt(nexti)=='\n') i++; // process cr/lf pair as one line break
-					appendable.append("<br />"); // add line break
-					continue;
-				} else {
-					spaceCount=TAB_LENGTH;
-				}
-			} else {
-				spaceCount=1;
-			}
-			while (nexti<endPos) {
-				ch=unencodedText.charAt(nexti);
-				if (ch==' ')
-					spaceCount+=1;
-				else if (ch=='\t')
-					spaceCount+=TAB_LENGTH;
-				else
-					break;
-				nexti++;
-			}
-			if (spaceCount==1) {
-				// handle the very common case of a single character to improve efficiency slightly
-				appendable.append(' ');
-				continue;
-			}
-			if (spaceCount%2==1) appendable.append(' '); // fist character is a space if we have an odd number of spaces
-			while (spaceCount>=2) {
-				appendable.append("&nbsp; "); // use alternating &nbsp; and spaces to keep original number of spaces
-				spaceCount-=2;
-			}
-			// note that the last character is never a nbsp, so that word wrapping won't result in a nbsp before the first character in a line
-			i=nexti-1; // minus 1 because top level for loop will add it again
-		}
+		for (int i=beginPos; i<endPos; i++) appendEncode(appendable,unencodedText.charAt(i),insideAttributeValue);
 		return appendable;
 	}
 
-	private static final boolean appendEncodeCheckForWhiteSpaceFormatting(final Appendable appendable, char ch, final boolean whiteSpaceFormatting) throws IOException {
-		final String characterEntityReferenceName=CharacterEntityReference.getName(ch);
-		if (characterEntityReferenceName!=null) {
-			if (ch=='\'') {
-				if (Config.IsApostropheEncoded)
-					appendable.append("&#39;");
-				else
-					appendable.append(ch);
-			} else {
+	private static final Appendable appendEncode(final Appendable appendable, final char ch, final boolean insideAttributeValue) throws IOException {
+		if (Config.CurrentCharacterReferenceEncodingBehaviour.isEncoded(ch,insideAttributeValue)) {
+			final String characterEntityReferenceName=CharacterEntityReference.getName(ch);
+			if (characterEntityReferenceName!=null && ch!='\'') {
 				CharacterEntityReference.appendCharacterReferenceString(appendable,characterEntityReferenceName);
+			} else {
+				appendDecimalCharacterReferenceString(appendable,ch);
 			}
-		} else if (ch>127) {
-			appendDecimalCharacterReferenceString(appendable,ch);
-		} else if (!(whiteSpaceFormatting && isWhiteSpace(ch))) {
-			appendable.append(ch);
 		} else {
-			return false;
+			appendable.append(ch);
 		}
-		return true;
+		return appendable;
 	}
 
 	static CharacterReference getPrevious(final Source source, final int pos) {
@@ -725,10 +737,10 @@ public abstract class CharacterReference extends Segment {
 		}
 	}
 
-	private static Appendable appendDecode(final Appendable appendable, final CharSequence encodedText, int pos, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) throws IOException {
+	private static Appendable appendDecode(final Appendable appendable, final CharSequence encodedText, final int searchBegin, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) throws IOException {
+		if (encodedText instanceof Segment) return appendDecode(appendable,(Segment)encodedText,searchBegin,insideAttributeValue,convertNonBreakingSpaces);
 		final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings=Config.CurrentCompatibilityMode.getUnterminatedCharacterReferenceSettings(insideAttributeValue);
-		int lastEnd=0;
-		final StreamedSource streamedSource=new StreamedSource(encodedText).setHandleTags(false).setUnterminatedCharacterReferenceSettings(unterminatedCharacterReferenceSettings).setSearchBegin(pos);
+		final StreamedSource streamedSource=new StreamedSource(encodedText).setHandleTags(false).setUnterminatedCharacterReferenceSettings(unterminatedCharacterReferenceSettings).setSearchBegin(searchBegin);
 		for (Segment segment : streamedSource) {
 			if (segment instanceof CharacterReference) {
 				((CharacterReference)segment).appendCharTo(appendable,convertNonBreakingSpaces);
@@ -741,6 +753,28 @@ public abstract class CharacterReference extends Segment {
 		return appendable;
 	}
 
+	private static Appendable appendDecode(final Appendable appendable, final Segment segment, final int searchBegin, final boolean insideAttributeValue, final boolean convertNonBreakingSpaces) throws IOException {
+		final Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings=Config.CurrentCompatibilityMode.getUnterminatedCharacterReferenceSettings(insideAttributeValue);
+  	final Source source=segment.source;
+  	final ParseText parseText=source.getParseText();
+  	final int end=segment.getEnd();
+  	int begin=segment.getBegin();
+		int pos=parseText.indexOf('&',begin+searchBegin,end);
+		while (pos!=-1) {
+			final CharacterReference characterReference=CharacterReference.construct(source,pos,unterminatedCharacterReferenceSettings);
+			if (characterReference!=null) {
+				appendable.append(source.substring(begin,pos)); // Don't use appendable.append(source,begin,pos) as it checks source.length() which may throw an exception when using StreamedSource.
+				characterReference.appendCharTo(appendable,convertNonBreakingSpaces);
+				begin=characterReference.getEnd();
+				pos=parseText.indexOf('&',begin,end);
+			} else {
+				pos=parseText.indexOf('&',pos+1,end);
+			}
+		}
+		appendable.append(source.substring(begin,end));
+		return appendable;
+	}
+
 	// pinched from http://svn.apache.org/repos/asf/abdera/java/trunk/dependencies/i18n/src/main/java/org/apache/abdera/i18n/text/CharUtils.java
 	private static char getHighSurrogate(int codePoint) {
 		return (char)((0xD800 - (0x10000 >> 10)) + (codePoint >> 10));
