@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -84,8 +88,8 @@ public abstract class FormControl extends Segment {
 			if (typeAttributeValue==null) return new InputFormControl(element,FormControlType.TEXT);
 			FormControlType formControlType=FormControlType.getFromInputElementType(typeAttributeValue);
 			if (formControlType==null) {
-				if (formControlType.isNonFormControl(typeAttributeValue)) return null;
-				if (element.source.logger.isInfoEnabled()) element.source.logger.info(element.source.getRowColumnVector(element.begin).appendTo(new StringBuilder(200)).append(": INPUT control with unrecognised type \"").append(typeAttributeValue).append("\" assumed to be type \"text\"").toString());
+				if (FormControlType.isNonFormControl(typeAttributeValue)) return null;
+				if (element.source.logger.isErrorEnabled()) element.source.logger.error(element.source.getRowColumnVector(element.begin).appendTo(new StringBuilder(200)).append(": INPUT control with unrecognised type \"").append(typeAttributeValue).append("\" assumed to be type \"text\"").toString());
 				formControlType=FormControlType.TEXT;
 			}
 			switch (formControlType) {
@@ -142,7 +146,7 @@ public abstract class FormControl extends Segment {
 	 * from the source document, regardless of the current setting of the static
 	 * {@link Config#CurrentCompatibilityMode}<code>.</code>{@link Config.CompatibilityMode#isFormFieldNameCaseInsensitive() FormFieldNameCaseInsensitive} property.
 	 *
-	 * @return the <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> of the control.
+	 * @return the <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> of the control, or <code>null</code> if the <code>name</code> attribute is missing.
 	 */
 	public final String getName() {
 		return name;
@@ -533,7 +537,8 @@ public abstract class FormControl extends Segment {
 
 	public String getDebugInfo() {
 		final StringBuilder sb=new StringBuilder();
-		sb.append(formControlType).append(" name=\"").append(name).append('"');
+		sb.append(formControlType);
+		if (name!=null) sb.append(" name=\"").append(name).append('"');
 		if (elementContainer.predefinedValue!=null) sb.append(" PredefinedValue=\"").append(elementContainer.predefinedValue).append('"');
 		sb.append(" - ").append(getElement().getDebugInfo());
 		return sb.toString();
@@ -596,7 +601,7 @@ public abstract class FormControl extends Segment {
 			} else {
 				replaceAttributesInOutputDocumentIfModified(outputDocument);
 				if (value!=UNCHANGED)
-					outputDocument.replace(getElement().getContent(),CharacterReference.encode(value));
+					outputDocument.replace(getElement().getContent(),CharacterReference.encode(value,false));
 			}
 		}
 		private String getValue() {
@@ -610,7 +615,7 @@ public abstract class FormControl extends Segment {
 			super(element,formControlType,true);
 			if (elementContainer.predefinedValue==null) {
 				elementContainer.predefinedValue=CHECKBOX_NULL_DEFAULT_VALUE;
-				if (element.source.logger.isInfoEnabled()) element.source.logger.info(element.source.getRowColumnVector(element.begin).appendTo(new StringBuilder(200)).append(": compulsory \"value\" attribute of ").append(formControlType).append(" control \"").append(name).append("\" is missing, assuming the value \"").append(CHECKBOX_NULL_DEFAULT_VALUE).append('"').toString());
+				if (element.source.logger.isErrorEnabled()) element.source.logger.error(element.source.getRowColumnVector(element.begin).appendTo(new StringBuilder(200)).append(": compulsory \"value\" attribute of ").append(formControlType).append(" control \"").append(name).append("\" is missing, assuming the value \"").append(CHECKBOX_NULL_DEFAULT_VALUE).append('"').toString());
 			}
 		}
 		public boolean setValue(final String value) {
@@ -780,10 +785,15 @@ public abstract class FormControl extends Segment {
 				Attribute.appendHTML(sb,attributeName,attributeValue);
 			}
 			sb.append('>');
-			if (text==null || text.length()==0)
+			if (text==null || text.length()==0) {
 				sb.append(FormControlOutputStyle.ConfigDisplayValue.EmptyHTML);
-			else
-				CharacterReference.appendEncode(sb,text,whiteSpaceFormatting);
+			} else {
+				if (whiteSpaceFormatting) {
+					sb.append(CharacterReference.encodeWithWhiteSpaceFormatting(text));
+				} else {
+					CharacterReference.appendEncode(sb,text,false);
+				}
+			}
 		} catch (IOException ex) {throw new RuntimeException(ex);} // never happens
 		sb.append(EndTagType.START_DELIMITER_PREFIX).append(FormControlOutputStyle.ConfigDisplayValue.ElementName).append('>');
 		return sb.toString();
@@ -820,14 +830,10 @@ public abstract class FormControl extends Segment {
 	private void verifyName() {
 		if (formControlType.isSubmit()) return;
 		String missingOrBlank;
-		if (name==null) {
-			missingOrBlank="missing";
-		} else {
-			if (name.length()!=0) return;
-			missingOrBlank="blank";
-		}
+		if (name==null) return;
+		if (name.length()!=0) return;
 		final Source source=getElement().source;
-		if (source.logger.isInfoEnabled()) source.logger.info(getElement().source.getRowColumnVector(getElement().begin).appendTo(new StringBuilder(200)).append(": compulsory \"name\" attribute of ").append(formControlType).append(" control is ").append(missingOrBlank).toString());
+		if (source.logger.isErrorEnabled()) source.logger.error(getElement().source.getRowColumnVector(getElement().begin).appendTo(new StringBuilder(200)).append(": compulsory \"name\" attribute of ").append(formControlType).append(" control is empty").toString());
 	}
 
 	private static final void addValueTo(final Collection<String> collection, final String value) {
