@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -366,8 +370,9 @@ public enum FormControlType {
 
 	/**
 	 * The form control type given to a <a href="#text-input-control">text input</a> control implemented using an
-	 * <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#edef-INPUT">INPUT</a></code> element with attribute
-	 * <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-type-INPUT">type</a>="text"</code>.
+	 * <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#edef-INPUT">INPUT</a></code> element with a
+	 * <code><a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-type-INPUT">type</a></code> attribute value of
+	 * <code>text</code>, <code>date</code>, <code>datetime</code>, <code>datetime-local</code>, <code>month</code>, <code>time</code>, <code>week</code>, <code>number</code>, <code>range</code>, <code>email</code>, <code>url</code>, <code>search</code>, <code>tel</code>, or <code>color</code>.
 	 * <p>
 	 * <dl>
 	 *  <dt>Example:</dt>
@@ -414,6 +419,19 @@ public enum FormControlType {
 		INPUT_ELEMENT_TYPE_MAP.put("radio",RADIO);
 		INPUT_ELEMENT_TYPE_MAP.put("submit",SUBMIT);
 		INPUT_ELEMENT_TYPE_MAP.put("text",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("date",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("datetime",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("datetime-local",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("month",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("time",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("week",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("number",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("range",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("email",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("url",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("search",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("tel",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("color",TEXT);
 		// The following INPUT element "type" attributes do not produce a form control:
 		NON_FORM_CONTROL_TYPE_ATTRIBUTE_SET.add("button");
 		NON_FORM_CONTROL_TYPE_ATTRIBUTE_SET.add("reset");
