@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -564,7 +568,7 @@ public final class FormFields extends AbstractCollection<FormField> {
 	 * Returns a list of all the {@linkplain FormField#getFormControls() constituent form controls} from all the {@linkplain FormField form fields} in this collection.
 	 * @return a list of all the {@linkplain FormField#getFormControls() constituent form controls} from all the {@linkplain FormField form fields} in this collection.
 	 */
-	public List getFormControls() {
+	public List<FormControl> getFormControls() {
 		return formControls;
 	}
 
@@ -629,7 +633,7 @@ public final class FormFields extends AbstractCollection<FormField> {
 
 	void add(final FormControl formControl, final String predefinedValue, String fieldName) {
 		if (Config.CurrentCompatibilityMode.isFormFieldNameCaseInsensitive()) fieldName=fieldName.toLowerCase();
-		FormField formField=(FormField)map.get(fieldName);
+		FormField formField=map.get(fieldName);
 		if (formField==null) {
 			formField=new FormField(fieldName);
 			map.put(formField.getName(),formField);
