@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -26,9 +30,18 @@ import java.io.*;
 final class LoggerProviderSTDERR implements LoggerProvider {
 	public static final LoggerProvider INSTANCE=new LoggerProviderSTDERR();
 	
+	private static volatile Logger sourceLogger=null;
+	
 	private LoggerProviderSTDERR() {}
 
 	public Logger getLogger(final String name) {
 		return new WriterLogger(new OutputStreamWriter(System.err),name);
 	}
+
+	public Logger getSourceLogger() {
+		// sourceLogger is declared volatile so can be safely used by multiple threads without synchronization.
+		// The worst that can happen is that it is set more than once, but that shouldn't matter.
+		if (sourceLogger==null) sourceLogger=getLogger(Source.PACKAGE_NAME);
+		return sourceLogger;
+	}
 }
