@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -342,13 +346,23 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	public List<StartTag> getAllStartTags(String name) {
 		if (name==null) return getAllStartTags();
 		final boolean isXMLTagName=Tag.isXMLName(name);
+		name=name.toLowerCase();
 		StartTag startTag=checkEnclosure(StartTag.getNext(source,begin,name,StartTagType.NORMAL,isXMLTagName));
 		if (startTag==null) return Collections.emptyList();
 		final ArrayList<StartTag> list=new ArrayList<StartTag>();
-		do {
+		if (source.wasFullSequentialParseCalled() && isXMLTagName) {
 			list.add(startTag);
-			startTag=checkEnclosure(StartTag.getNext(source,startTag.begin+1,name,StartTagType.NORMAL,isXMLTagName));
-		} while (startTag!=null);
+			while (true) {
+				startTag=checkEnclosure((StartTag)startTag.getNextTag(StartTagType.NORMAL));
+				if (startTag==null) break;
+				if (startTag.name.equals(name) || (startTag.name.startsWith(name) && startTag.isPartialNameSearchMatch(name))) list.add(startTag);
+			}
+		} else {
+			do {
+				list.add(startTag);
+				startTag=checkEnclosure(StartTag.getNext(source,startTag.begin+1,name,StartTagType.NORMAL,isXMLTagName));
+			} while (startTag!=null);
+		}
 		return list;
 	}
 
@@ -367,10 +381,19 @@ public class Segment implements Comparable<Segment>, CharSequence {
 		StartTag startTag=checkEnclosure(source.getNextStartTag(begin,attributeName,value,valueCaseSensitive));
 		if (startTag==null) return Collections.emptyList();
 		final ArrayList<StartTag> list=new ArrayList<StartTag>();
-		do {
+		if (source.wasFullSequentialParseCalled()) {
 			list.add(startTag);
-			startTag=checkEnclosure(source.getNextStartTag(startTag.begin+1,attributeName,value,valueCaseSensitive));
-		} while (startTag!=null);
+			while (true) {
+				startTag=checkEnclosure(startTag.getNextStartTag());
+				if (startTag==null) break;
+				if (startTag.hasAttribute(attributeName,value,valueCaseSensitive)) list.add(startTag);
+			}
+		} else {
+			do {
+				list.add(startTag);
+				startTag=checkEnclosure(source.getNextStartTag(startTag.begin+1,attributeName,value,valueCaseSensitive));
+			} while (startTag!=null);
+		}
 		return list;
 	}
 
@@ -391,10 +414,19 @@ public class Segment implements Comparable<Segment>, CharSequence {
 		StartTag startTag=checkEnclosure(source.getNextStartTag(begin,attributeName,valueRegexPattern));
 		if (startTag==null) return Collections.emptyList();
 		final ArrayList<StartTag> list=new ArrayList<StartTag>();
-		do {
+		if (source.wasFullSequentialParseCalled()) {
 			list.add(startTag);
-			startTag=checkEnclosure(source.getNextStartTag(startTag.begin+1,attributeName,valueRegexPattern));
-		} while (startTag!=null);
+			while (true) {
+				startTag=checkEnclosure(startTag.getNextStartTag());
+				if (startTag==null) break;
+				if (startTag.hasAttribute(attributeName,valueRegexPattern)) list.add(startTag);
+			}
+		} else {
+			do {
+				list.add(startTag);
+				startTag=checkEnclosure(source.getNextStartTag(startTag.begin+1,attributeName,valueRegexPattern));
+			} while (startTag!=null);
+		}
 		return list;
 	}
 
@@ -585,6 +617,7 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	 *  <tr><th>HTML element name<th>Attribute name
 	 *  <tr><td>{@link HTMLElementName#A A}<td>href
 	 *  <tr><td>{@link HTMLElementName#APPLET APPLET}<td>codebase
+	 *  <tr><td>{@link HTMLElementName#APPLET APPLET}<td>archive
 	 *  <tr><td>{@link HTMLElementName#AREA AREA}<td>href
 	 *  <tr><td>{@link HTMLElementName#BASE BASE}<td>href
 	 *  <tr><td>{@link HTMLElementName#BLOCKQUOTE BLOCKQUOTE}<td>cite
@@ -603,6 +636,7 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	 *  <tr><td>{@link HTMLElementName#INPUT INPUT}<td>usemap
 	 *  <tr><td>{@link HTMLElementName#INS INS}<td>cite
 	 *  <tr><td>{@link HTMLElementName#LINK LINK}<td>href
+	 *  <tr><td>{@link HTMLElementName#OBJECT OBJECT}<td>archive
 	 *  <tr><td>{@link HTMLElementName#OBJECT OBJECT}<td>classid
 	 *  <tr><td>{@link HTMLElementName#OBJECT OBJECT}<td>codebase
 	 *  <tr><td>{@link HTMLElementName#OBJECT OBJECT}<td>data
@@ -626,16 +660,17 @@ public class Segment implements Comparable<Segment>, CharSequence {
 
 	/**
 	 * Returns a list of all <a target="_blank" href="http://en.wikipedia.org/wiki/URI">URI</a> {@linkplain Segment segments}
-	 * inside <code>style</code> attribute values {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * inside the CSS of {@link HTMLElementName#STYLE STYLE} elements and <code>style</code> attribute values
+	 * {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
-	 * If this segment does not contain any tags, the entire segment is assumed to be a <code>style</code> attribute value.
+	 * If this segment does not contain any tags, the entire segment is assumed to be CSS.
 	 * <p>
-	 * The URI segments are found by searching the <code>style</code> attribute values for the functional notation "<code>url()</code>" as described in
+	 * The URI segments are found by searching the CSS for the functional notation "<code>url()</code>" as described in
 	 * <a target="_blank" href="http://www.w3.org/TR/CSS2/syndata.html#uri">section 4.3.4 of the CSS2 specification</a>.
 	 * <p>
 	 * The segments are returned in order of appearance.
 	 *
-	 * @return a list of all <a target="_blank" href="http://en.wikipedia.org/wiki/URI">URI</a> {@linkplain Segment segments} inside <code>style</code> attribute values {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * @return a list of all <a target="_blank" href="http://en.wikipedia.org/wiki/URI">URI</a> {@linkplain Segment segments} inside {@link HTMLElementName#STYLE STYLE} elements and <code>style</code> attribute values {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * @see #getURIAttributes()
 	 */
 	public List<Segment> getStyleURISegments() {
@@ -716,7 +751,7 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	/**
 	 * Returns the first {@link StartTag} with the specified class that is {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
-	 * This is functionally equivalent to {@link #getAllStartTagsByClass(String) getAllStartTagsByClass(className)}<code>.iterator().next()</code>,
+	 * This is functionally equivalent to {@link #getAllStartTagsByClass(String) getAllStartTagsByClass(className)}<code>.get(0)</code>,
 	 * but does not search beyond the first start tag and returns <code>null</code> if no such start tag exists.
 	 *
 	 * @param className  the class name (case sensitive) to search for, must not be <code>null</code>.
@@ -762,6 +797,7 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	public final Element getFirstElement(String name) {
 		if (name==null) return getFirstElement();
 		final boolean isXMLTagName=Tag.isXMLName(name);
+		name=name.toLowerCase();
 		StartTag startTag=checkEnclosure(StartTag.getNext(source,begin,name,StartTagType.NORMAL,isXMLTagName));
 		while (startTag!=null) {
 			final Element element=startTag.getElement();
@@ -821,7 +857,7 @@ public class Segment implements Comparable<Segment>, CharSequence {
 	/**
 	 * Returns the first {@link Element} with the specified class that is {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
-	 * This is functionally equivalent to {@link #getAllElementsByClass(String) getAllElementsByClass(className)}<code>.iterator().next()</code>,
+	 * This is functionally equivalent to {@link #getAllElementsByClass(String) getAllElementsByClass(className)}<code>.get(0)</code>,
 	 * but does not search beyond the first enclosed element and returns <code>null</code> if no such element exists.
 	 * <p>
 	 * If this segment is itself an {@link Element} with the specified class, this element is returned.
@@ -1024,6 +1060,15 @@ public class Segment implements Comparable<Segment>, CharSequence {
 		return false;
 	}
 
+	/**
+	 * Returns a {@link RowColumnVector} object representing the row and column number of the {@linkplain #getBegin() start} of this segment in the source document.
+	 * @return a {@link RowColumnVector} object representing the row and column number of the {@linkplain #getBegin() start} of this segment in the source document.
+	 * @see Source#getRowColumnVector(int pos)
+	 */
+	public RowColumnVector getRowColumnVector() {
+		return source.getRowColumnVector(begin);
+	}
+
 	/**
 	 * Returns a string representation of this object useful for debugging purposes.
 	 * @return a string representation of this object useful for debugging purposes.
