@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -170,7 +174,7 @@ public final class StartTag extends Tag {
 				if (endTag.element!=Element.NOT_CACHED) {
 					// This is presumably impossible, except in certain circumstances where the cache was cleared, such as if the parser decides to do a full sequential parse after some tags have already been found.
 					// If the existing element and the current element are not the same, log it.
-					if (source.logger.isInfoEnabled() && !element.equals(endTag.element)) source.logger.info(source.getRowColumnVector(endTag.begin).appendTo(new StringBuilder(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); 
+					if (source.logger.isErrorEnabled() && !element.equals(endTag.element)) source.logger.error(source.getRowColumnVector(endTag.begin).appendTo(new StringBuilder(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); 
 				}
 				endTag.element=element;
 			}
@@ -408,8 +412,8 @@ public final class StartTag extends Tag {
 	 * The tidying of the tag is carried out as follows:
 	 * <ul>
 	 *  <li>if this start tag is of a {@linkplain #getStartTagType() type} that does not {@linkplain StartTagType#hasAttributes() have attributes},
-	 *   then the original {@linkplain Segment#toString() source text} of the enture tag is returned.
-	 *  <li>if this start tag contain any {@linkplain TagType#isServerTag() server tags} outside of an attribute value,
+	 *   then the original {@linkplain Segment#toString() source text} of the entire tag is returned.
+	 *  <li>if this start tag contains any {@linkplain TagType#isServerTag() server tags} outside of an attribute value,
 	 *   then the original {@linkplain Segment#toString() source text} of the entire tag is returned.
 	 *  <li>name converted to lower case if the <code>toXHTML</code> argument is <code>true</code> and this is a {@linkplain StartTagType#NORMAL normal} start tag
 	 *  <li>attributes separated by a single space
@@ -561,7 +565,7 @@ public final class StartTag extends Tag {
 			if (startTagType==StartTagType.NORMAL && HTMLElements.END_TAG_REQUIRED_NESTING_FORBIDDEN_SET.contains(name)) {
 				final StartTag nextStartTag=source.getNextStartTag(end,name);
 				if (nextStartTag==null || nextStartTag.begin>nextEndTag.begin) return nextEndTag;
-				if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append("StartTag at ")).append(" missing required end tag - invalid nested start tag encountered before end tag").toString());
+				if (source.logger.isErrorEnabled()) source.logger.error(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append("StartTag at ")).append(" missing required end tag - invalid nested start tag encountered before end tag").toString());
 				// Terminate the element at the start of the invalidly nested start tag.
 				// This is how IE and Mozilla treat illegally nested A elements, but other elements may vary.
 				return new EndTag(source,nextStartTag.begin,nextStartTag.begin,EndTagType.NORMAL,name);
@@ -569,7 +573,7 @@ public final class StartTag extends Tag {
 			final Segment[] getResult=getEndTag(nextEndTag,checkForEmptyElementTag,Tag.isXMLName(name));
 			if (getResult!=null) return (EndTag)getResult[0];
 		}
-		if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append("StartTag at ")).append(" missing required end tag").toString());
+		if (source.logger.isErrorEnabled()) source.logger.error(source.getRowColumnVector(begin).appendTo(new StringBuilder(200).append("StartTag at ")).append(" missing required end tag").toString());
 		return null;
 	}
 
@@ -578,7 +582,7 @@ public final class StartTag extends Tag {
 		while (pos<source.end) {
 			final Tag tag=Tag.getNextTag(source,pos);
 			if (tag==null) break;
-			Set terminatingTagNameSet;
+			Set<String> terminatingTagNameSet;
 			if (tag instanceof EndTag) {
 				if (tag.name==name) return (EndTag)tag;
 				terminatingTagNameSet=terminatingTagNameSets.TerminatingEndTagNameSet;
@@ -612,43 +616,58 @@ public final class StartTag extends Tag {
 	static StartTag getPrevious(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType, final boolean isXMLTagName) {
 		// searchName is already in lower case, but may be null
 		// searchStartTagType must not be null
-		// isXMLTagName is only used if searchStartTagType==StartTagType.NORMAL
+		// searchStartTagType may be something other than NORMAL together with a searchName when called from Source.getPreviousStartTag(pos,name,startTagType) (and possibly other situations)
+		// isXMLTagName is only used if searchStartTagType==StartTagType.NORMAL, otherwise it is always true.
 		if (searchName==null) return (StartTag)source.getPreviousTag(pos,searchStartTagType);
-		final String startDelimiter=getStartDelimiter(searchName);
-		try {
-			final ParseText parseText=source.getParseText();
-			int begin=pos;
-			do {
-				begin=parseText.lastIndexOf(startDelimiter,begin);
-				if (begin==-1) return null;
-				final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
-				if (startTag==null) continue; // keep looking if it wasn't a start tag
-				if (searchStartTagType!=startTag.getStartTagType()) {
-					// The start tag is of the wrong type.  The only case in which we want to return it is if
-					// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
-					// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
-					if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
-				}
-				if (startTag.getStartTagType().isNameAfterPrefixRequired() && startTag.getName().length()>searchName.length()) {
-					// The name of the start tag is longer than the search name, and the type of tag indicates 
-					// that we are probably looking for an exact match.
-					// (eg searchName="a", startTag.name="applet" -> reject)
-					// We only require an exact match if the last character of the search name is part of the name, as the
-					// search name might be just the prefix of a server tag.
-					// (eg searchName="?", startTag.name="?abc" -> accept, but searchName="?a", startTag.name="?abc" -> reject)
-					// The only exception to this is if the last character of the search name is a colon (which also forms part of
-					// the name), but signifies that we want to search on the entire namespace.
-					// (eg searchName="o:", startTag.name="o:p" -> accept)
-					char lastSearchNameChar=searchName.charAt(searchName.length()-1);
-					if (lastSearchNameChar!=':' && isXMLNameChar(lastSearchNameChar)) continue;
-				}
-				return startTag;
-			} while ((begin-=2)>=0);
-		} catch (IndexOutOfBoundsException ex) {
-			// this should never happen during a get previous operation so rethrow it:
-			throw ex;
+		if (source.wasFullSequentialParseCalled() && isXMLTagName) {
+			// this cache search only finds tags of the specified type, so don't do it if !isXMLTagName because the user might be looking for UNREGISTERED tags using name search.
+			StartTag startTag=(StartTag)Tag.getPreviousTag(source,pos,searchStartTagType);
+			while (true) {
+				if (startTag==null) return null;
+				if (startTag.name.equals(searchName)) return startTag;
+				if (startTag.name.startsWith(searchName) && startTag.isPartialNameSearchMatch(searchName)) return startTag;
+				if (startTag.name.length()<searchName.length() && source.getParseText().containsAt(searchName,startTag.begin+searchStartTagType.startDelimiterPrefix.length())) return startTag;
+				startTag=(StartTag)startTag.getPreviousTag(searchStartTagType);
+			}
+		} else {
+			final String startDelimiter=getStartDelimiter(searchName);
+			try {
+				final ParseText parseText=source.getParseText();
+				int begin=pos;
+				do {
+					begin=parseText.lastIndexOf(startDelimiter,begin);
+					if (begin==-1) return null;
+					final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
+					if (startTag==null) continue; // keep looking if it wasn't a start tag
+					if (searchStartTagType!=startTag.getStartTagType()) {
+						// The start tag is of the wrong type.  The only case in which we want to return it is if
+						// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
+						// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
+						if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
+					}
+					if (startTag.getName().length()>searchName.length() && !startTag.isPartialNameSearchMatch(searchName)) continue;
+					// don't reject if startTag.getName().length()<searchName.length() as there is no reason they shouldn't match. e.g. searching for "div " finds "div" tags, and searching for "#unregistered" finds unregistered tags which have name "".
+					return startTag;
+				} while ((begin-=2)>=0);
+			} catch (IndexOutOfBoundsException ex) {
+				// this should never happen during a get previous operation so rethrow it:
+				throw ex;
+			}
+			return null;
 		}
-		return null;
+	}
+
+	final boolean isPartialNameSearchMatch(final String searchName) {
+		// Assume the search tag name matches the start of this tag name.
+		// Note there is no need to test for startTagType.isNameAfterPrefixRequired().
+		// Accept the partial match if the last character of the search name is not a valid XML name character, as it indicates the search name 
+		// is just the prefix of a server tag. ***** check these comments!!!
+		// (eg searchName="?", name="?abc" -> accept, but searchName="?a", name="?abc" -> reject)
+		// (eg searchName="a", name="applet" -> reject)
+		// Also accept the partial match if the last character of the search name is a colon (which also forms part of the name), but signifies that we want to search on the entire namespace.
+		// (eg searchName="o:", name="o:p" -> accept)
+		char lastSearchNameChar=searchName.charAt(searchName.length()-1);
+		return lastSearchNameChar==':' || !isXMLNameChar(lastSearchNameChar);
 	}
 
 	static StartTag getNext(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType) {
@@ -658,54 +677,57 @@ public final class StartTag extends Tag {
 	static StartTag getNext(final Source source, final int pos, final String searchName, final StartTagType searchStartTagType, final boolean isXMLTagName) {
 		// searchName is already in lower case, but may be null
 		// searchStartTagType must not be null
-		// isXMLTagName is only used if searchStartTagType==StartTagType.NORMAL
+		// searchStartTagType may be something other than NORMAL together with a searchName when called from Source.getNextStartTag(pos,name,startTagType) or when searching for nested start tags during internal end tag search.
+		// isXMLTagName is only used if searchStartTagType==StartTagType.NORMAL, otherwise it is always true.
 		if (searchName==null) return (StartTag)source.getNextTag(pos,searchStartTagType);
-		final String startDelimiter=getStartDelimiter(searchName);
-		try {
-			final ParseText parseText=source.getParseText();
-			int begin=pos;
-			do {
-				begin=parseText.indexOf(startDelimiter,begin);
-				if (begin==-1) return null;
-				final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
-				if (startTag==null) continue; // keep looking if it wasn't a start tag
-				if (searchStartTagType!=startTag.getStartTagType()) {
-					// The start tag is of the wrong type.  The only case in which we want to return it is if
-					// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
-					// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
-					if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
-				}
-				if (startTag.getStartTagType().isNameAfterPrefixRequired() && startTag.getName().length()>searchName.length()) {
-					// The name of the start tag is longer than the search name, and the type of tag indicates 
-					// that we are probably looking for an exact match.
-					// (eg searchName="a", startTag.name="applet" -> reject)
-					// We only require an exact match if the last character of the search name is part of the name, as the
-					// search name might be just the prefix of a server tag.
-					// (eg searchName="?", startTag.name="?abc" -> accept, but searchName="?a", startTag.name="?abc" -> reject)
-					// The only exception to this is if the last character of the search name is a colon (which also forms part of
-					// the name), but signifies that we want to search on the entire namespace.
-					// (eg searchName="o:", startTag.name="o:p" -> accept)
-					char lastSearchNameChar=searchName.charAt(searchName.length()-1);
-					if (lastSearchNameChar!=':' && isXMLNameChar(lastSearchNameChar)) continue;
-				}
-				return startTag;
-			} while ((begin+=1)<source.end);
-		} catch (IndexOutOfBoundsException ex) {
-			// this should only happen when the end of file is reached in the middle of a tag.
-			// we don't have to do anything to handle it as there are no more tags anyway.
+		if (source.wasFullSequentialParseCalled() && isXMLTagName) {
+			// This cache search only finds tags of the specified type, so don't do it if !isXMLTagName because the user might be looking for UNREGISTERED tags using standard name search.
+			// We still might reach here if the user called Source.getNextStartTag(pos,name,startTagType)
+			StartTag startTag=(StartTag)Tag.getNextTag(source,pos,searchStartTagType);
+			while (true) {
+				if (startTag==null) return null;
+				if (startTag.name.equals(searchName)) return startTag;
+				if (startTag.name.startsWith(searchName) && startTag.isPartialNameSearchMatch(searchName)) return startTag;
+				if (startTag.name.length()<searchName.length() && source.getParseText().containsAt(searchName,startTag.begin+searchStartTagType.startDelimiterPrefix.length())) return startTag;
+				startTag=(StartTag)startTag.getNextTag(searchStartTagType);
+			}
+		} else {
+			final String startDelimiter=getStartDelimiter(searchName);
+			try {
+				final ParseText parseText=source.getParseText();
+				int begin=pos;
+				do {
+					begin=parseText.indexOf(startDelimiter,begin);
+					if (begin==-1) return null;
+					final StartTag startTag=(StartTag)Tag.getTagAt(source,begin,false);
+					if (startTag==null) continue; // keep looking if it wasn't a start tag
+					if (searchStartTagType!=startTag.getStartTagType()) {
+						// The start tag is of the wrong type.  The only case in which we want to return it is if
+						// we are looking for a normal start tag, the found start tag is unregistered, and the search name is NOT a valid XML name.
+						// This allows users to search for some types of unregistered tags by name rather than having to register custom tag types.
+						if (searchStartTagType!=StartTagType.NORMAL || isXMLTagName || !startTag.isUnregistered()) continue;
+					}
+					if (startTag.getName().length()>searchName.length() && !startTag.isPartialNameSearchMatch(searchName)) continue;
+					// don't reject if startTag.getName().length()<searchName.length() as there is no reason they shouldn't match. e.g. searching for "div " finds "div" tags, and searching for "#unregistered" finds unregistered tags which have name "".
+					return startTag;
+				} while ((begin+=1)<source.end);
+			} catch (IndexOutOfBoundsException ex) {
+				// this should only happen when the end of file is reached in the middle of a tag.
+				// we don't have to do anything to handle it as there are no more tags anyway.
+			}
+			return null;
 		}
-		return null;
 	}
 
 	static StartTag getPrevious(final Source source, int pos) {
-		Tag tag=Tag.getPreviousTag(source,pos);
+		final Tag tag=Tag.getPreviousTag(source,pos);
 		if (tag==null) return null;
 		if (tag instanceof StartTag) return (StartTag)tag;
 		return tag.getPreviousStartTag();
 	}
 
 	static StartTag getNext(final Source source, int pos) {
-		Tag tag=Tag.getNextTag(source,pos);
+		final Tag tag=Tag.getNextTag(source,pos);
 		if (tag==null) return null;
 		if (tag instanceof StartTag) return (StartTag)tag;
 		return tag.getNextStartTag();
@@ -713,66 +735,91 @@ public final class StartTag extends Tag {
 
 	static StartTag getNext(final Source source, final int pos, final String attributeName, final String value, final boolean valueCaseSensitive) {
 		if (value==null || attributeName.length()==0) throw new IllegalArgumentException();
-		// Determine whether to perform the text search on the name or value:
-		// - perform the text search on the value if it is >= 3 chars long.
-		// - have to perform the text search on the name if the value is zero length.
-		// - perform the text search on the name if the name >= 3 chars long, otherwise on the value.
-		final String searchString=value.length()>=3 || (value.length()>0 && attributeName.length()<3) ? value : attributeName;
-		final ParseText parseText=source.getParseText();
-		int searchPos=pos;
-		while (searchPos<source.end) {
-			searchPos=parseText.indexOf(searchString.toLowerCase(),searchPos);
-			if (searchPos==-1) return null;
-			final Tag tag=source.getEnclosingTag(searchPos);
-			if (tag==null || !(tag instanceof StartTag)) {
-				searchPos++;
-				continue;
+		if (source.wasFullSequentialParseCalled()) {
+			StartTag startTag=source.getNextStartTag(pos);
+			while (true) {
+				if (startTag==null) return null;
+				if (startTag.hasAttribute(attributeName,value,valueCaseSensitive)) return startTag;
+				startTag=startTag.getNextStartTag();
 			}
-			if (tag.begin>=pos) {
-				final StartTag startTag=(StartTag)tag;
-				if (startTag.getAttributes()!=null) {
-					final String attributeValue=startTag.getAttributes().getValue(attributeName);
-					if (attributeValue!=null) {
-						if (value.equals(attributeValue)) return startTag;
-						if (value.equalsIgnoreCase(attributeValue)) {
-							if (!valueCaseSensitive) return startTag;
-							if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(searchPos).appendTo(new StringBuilder(200)).append(": StartTag with attribute ").append(attributeName).append("=\"").append(attributeValue).append("\" ignored during search because its case does not match search value \"").append(value).append('"').toString());
-						}
-					}
+		} else {
+			// Determine whether to perform the text search on the name or value:
+			// - perform the text search on the value if it is >= 3 chars long.
+			// - have to perform the text search on the name if the value is zero length.
+			// - perform the text search on the name if the name >= 3 chars long, otherwise on the value.
+			final String searchString=value.length()>=3 || (value.length()>0 && attributeName.length()<3) ? value : attributeName;
+			final ParseText parseText=source.getParseText();
+			int searchPos=pos;
+			while (searchPos<source.end) {
+				searchPos=parseText.indexOf(searchString.toLowerCase(),searchPos);
+				if (searchPos==-1) return null;
+				final Tag tag=source.getEnclosingTag(searchPos);
+				if (tag==null || !(tag instanceof StartTag)) {
+					searchPos++;
+					continue;
 				}
+				if (tag.begin>=pos) {
+					final StartTag startTag=(StartTag)tag;
+					if (startTag.hasAttribute(attributeName,value,valueCaseSensitive)) return startTag;
+				}
+				searchPos=tag.end;
 			}
-			searchPos=tag.end;
+			return null;
 		}
-		return null;
 	}
 
-	static StartTag getNext(final Source source, final int pos, final String attributeName, final Pattern regexPattern) {
-		if (attributeName==null || attributeName.length()==0) throw new IllegalArgumentException();
-		final String searchString=attributeName;
-		final ParseText parseText=source.getParseText();
-		int searchPos=pos;
-		while (searchPos<source.end) {
-			searchPos=parseText.indexOf(searchString.toLowerCase(),searchPos);
-			if (searchPos==-1) return null;
-			final Tag tag=source.getEnclosingTag(searchPos);
-			if (tag==null || !(tag instanceof StartTag)) {
-				searchPos++;
-				continue;
+	boolean hasAttribute(final String attributeName, final String value, final boolean valueCaseSensitive) {
+		if (attributes==null) return false;
+		final String attributeValue=attributes.getValue(attributeName);
+		if (attributeValue==null) return false;
+		if (value.equals(attributeValue)) return true;
+		if (!valueCaseSensitive || source.logger.isWarnEnabled()) {
+			if (value.equalsIgnoreCase(attributeValue)) {
+				if (!valueCaseSensitive) return true;
+				if (source.logger.isWarnEnabled()) source.logger.warn(getRowColumnVector().appendTo(new StringBuilder(200)).append(": StartTag with attribute ").append(attributeName).append("=\"").append(attributeValue).append("\" ignored during search because its case does not match search value \"").append(value).append('"').toString());
 			}
-			if (tag.begin>=pos) {
-				final StartTag startTag=(StartTag)tag;
-				if (startTag.getAttributes()!=null) {
-					final Attribute attribute=startTag.getAttributes().get(attributeName);
-					if (attribute!=null) {
-						if (regexPattern==null) return startTag;
-						final String attributeValue=attribute.getValue();
-						if (attributeValue!=null && regexPattern.matcher(attributeValue).matches()) return startTag;
-					}
-				}
-			}
-			searchPos=tag.end;
 		}
-		return null;
+		return false;
+	}
+
+	static StartTag getNext(final Source source, final int pos, final String attributeName, final Pattern valueRegexPattern) {
+		if (attributeName==null || attributeName.length()==0) throw new IllegalArgumentException();
+		if (source.wasFullSequentialParseCalled()) {
+			StartTag startTag=source.getNextStartTag(pos);
+			while (true) {
+				if (startTag==null) return null;
+				if (startTag.hasAttribute(attributeName,valueRegexPattern)) return startTag;
+				startTag=startTag.getNextStartTag();
+			}
+		} else {
+			final String searchString=attributeName;
+			final ParseText parseText=source.getParseText();
+			int searchPos=pos;
+			while (searchPos<source.end) {
+				searchPos=parseText.indexOf(searchString.toLowerCase(),searchPos);
+				if (searchPos==-1) return null;
+				final Tag tag=source.getEnclosingTag(searchPos);
+				if (tag==null || !(tag instanceof StartTag)) {
+					searchPos++;
+					continue;
+				}
+				if (tag.begin>=pos) {
+					final StartTag startTag=(StartTag)tag;
+					if (startTag.hasAttribute(attributeName,valueRegexPattern)) return startTag;
+				}
+				searchPos=tag.end;
+			}
+			return null;
+		}
+	}
+
+	boolean hasAttribute(final String attributeName, final Pattern valueRegexPattern) {
+		if (attributes==null) return false;
+		final Attribute attribute=attributes.get(attributeName);
+		if (attribute==null) return false;
+		if (valueRegexPattern==null) return true;
+		final String attributeValue=attribute.getValue();
+		return attributeValue!=null && valueRegexPattern.matcher(attributeValue).matches();
 	}
 
 	private Segment[] getEndTag(final EndTag nextEndTag, final boolean checkForEmptyElementTag, final boolean isXMLTagName) {
