@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -85,7 +89,7 @@ import java.net.URLConnection;
  * <p>
  * <style type="text/css">
  *  table#ParserComparison td, table#ParserComparison th {padding: 0px 5px 0px 5px}
- *  .checkmark {text-align: center; font-size: 12pt}
+ *  .checkmark {text-align: center !important; font-size: 12pt}
  * </style>
  * <table id="ParserComparison" class="bordered" cellspacing="0">
  *  <tr><th class="LabelColumn">Feature</th><th><code>StreamedSource</code></th><th><a target="_blank" href="http://en.wikipedia.org/wiki/StAX">StAX</a></th><th><a target="_blank" href="http://en.wikipedia.org/wiki/Simple_API_for_XML">SAX</a></th></tr>
@@ -118,8 +122,8 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 	private final StreamedText streamedText;
 	private final StreamedParseText streamedParseText;
 	private final Source source;
-	private final Closeable closeable; // internally created closeable object should be cleaned up internally.
-	private final boolean automaticClose;
+	private InternallyCreatedReaderFinalizer internallyCreatedReaderFinalizer=null;
+	private final Reader reader;
 	private boolean coalescing=false;
 	private boolean handleTags=true;
 	private Config.UnterminatedCharacterReferenceSettings unterminatedCharacterReferenceSettings=Config.CurrentCompatibilityMode.getUnterminatedCharacterReferenceSettings(false);
@@ -131,16 +135,16 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 	private static final boolean assumeNoNestedTags=false;
 	private static final Segment START_SEGMENT=new Segment(-1,-1);
 
-	private StreamedSource(final Reader reader, final boolean automaticClose, final String encoding, final String encodingSpecificationInfo, final String preliminaryEncodingInfo) throws IOException {
-		closeable=reader;
-		this.automaticClose=automaticClose;
+	private StreamedSource(final Reader reader, final boolean isInternallyCreatedReader, final String encoding, final String encodingSpecificationInfo, final String preliminaryEncodingInfo) throws IOException {
+		this.reader=reader;
+		if (isInternallyCreatedReader) internallyCreatedReaderFinalizer=new InternallyCreatedReaderFinalizer(reader);
 		streamedText=new StreamedText(reader);
 		streamedParseText=new StreamedParseText(streamedText);
 		source=new Source(streamedText,streamedParseText,encoding,encodingSpecificationInfo,preliminaryEncodingInfo);
 	}
 
-	private StreamedSource(final EncodingDetector encodingDetector, final boolean automaticClose) throws IOException {
-		this(encodingDetector.openReader(),automaticClose,encodingDetector.getEncoding(),encodingDetector.getEncodingSpecificationInfo(),encodingDetector.getPreliminaryEncoding()+": "+encodingDetector.getPreliminaryEncodingSpecificationInfo());
+	private StreamedSource(final EncodingDetector encodingDetector, final boolean isInternallyCreatedReader) throws IOException {
+		this(encodingDetector.openReader(),isInternallyCreatedReader,encodingDetector.getEncoding(),encodingDetector.getEncodingSpecificationInfo(),encodingDetector.getPreliminaryEncoding()+": "+encodingDetector.getPreliminaryEncodingSpecificationInfo());
 	}
 
 	/**
@@ -225,8 +229,7 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 	 * @param text  the source text.
 	 */
 	public StreamedSource(final CharSequence text) {
-		closeable=null;
-		automaticClose=false;
+		reader=null;
 		streamedText=new StreamedText(text);
 		streamedParseText=new StreamedParseText(streamedText);
 		source=new Source(text,streamedParseText,null,"Document specified encoding can not be determined automatically from a streamed source",null);
@@ -289,7 +292,19 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 	 * @throws IOException if an I/O error occurs.
 	 */
 	public void close() throws IOException {
-		if (closeable!=null) closeable.close();
+		if (reader==null) return; // implies internallyCreatedReaderFinalizer is also null
+		try {
+			reader.close();
+		} finally {
+			if (internallyCreatedReaderFinalizer!=null) {
+				internallyCreatedReaderFinalizer.setReaderToNull(); // Prevent attempt to close the reader a second time during finalization
+				internallyCreatedReaderFinalizer=null; // Allow GC to clean up the InternallyCreatedReaderFinalizer object even if references to this StreamedSource still exist. Probably a negligible optimisation.
+			}
+		}
+	}
+
+	private void closeInternallyCreatedReader() {
+		if (internallyCreatedReaderFinalizer!=null) try {close();} catch (IOException ex) {};
 	}
 
 	/**
@@ -548,15 +563,6 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 		return super.toString();
 	}
 
-	/**
-	 * Called by the garbage collector on an object when garbage collection determines that there are no more references to the object.
-	 * <p>
-	 * This implementation calls the {@link #close()} method if the underlying <code>Reader</code> or <code>InputStream</code> stream was created internally.
-	 */
-	protected void finalize() {
-		automaticClose();
-	}
-
 	StreamedSource setHandleTags(final boolean handleTags) {
 		this.handleTags=handleTags;
 		return this;
@@ -574,10 +580,6 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 		return this;
 	}
 
-	private void automaticClose() {
-		if (automaticClose) try {close();} catch (IOException ex) {}
-	}
-
 	private static boolean isXML(final Segment firstNonTextSegment) {
 		if (firstNonTextSegment==null || !(firstNonTextSegment instanceof Tag)) return false;
 		Tag tag=(Tag)firstNonTextSegment;
@@ -612,7 +614,9 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 			if (!hasNext()) throw new NoSuchElementException();
 			final Segment result=nextSegment;
 			nextSegment=(result==nextParsedSegment) ? Tag.NOT_CACHED : nextParsedSegment;
-			streamedText.setMinRequiredBufferBegin(result.end); // guaranteed not to be discarded until next call to loadNextParsedSegment()
+			// Can only set minimum required buffer begin to the start of this segment if it is a start tag, as the next tag may be inside it. Otherwise can set it to the end of this segment.
+			// Could probably optimise memory usage by adding another call to setMinRequiredBufferBegin in findNextParsedSegment as well. When it is searching beyond the end of the last tag it can be set to the end of the last tag. Haven't implemented this yet though as it would require thorough testing.
+			streamedText.setMinRequiredBufferBegin((result instanceof StartTag) ? result.begin : result.end); // guaranteed not to be discarded until next call to loadNextParsedSegment()
 			currentSegment=result;
 			return result;
 		}
@@ -636,6 +640,7 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 				final int searchEnd=coalescing ? streamedText.getEnd() : streamedText.getBufferOverflowPosition();
 				while (i<searchEnd) {
 					final char ch=streamedText.charAt(i);
+					if (ch==StreamedText.END_OF_STREAM && streamedText.atEndOfStream()) break; // checking ch==StreamedText.END_OF_STREAM first is superfluous but is probably faster than calling atEndOfStream() in the normal case.
 					if (ch=='&') {
 						if (i>=source.fullSequentialParseData[0]) { // do not handle character references inside tags or script elements
 							final CharacterReference characterReference=CharacterReference.construct(source,i,unterminatedCharacterReferenceSettings);
@@ -659,15 +664,35 @@ public final class StreamedSource implements Iterable<Segment>, Closeable {
 				}
 			} catch (BufferOverflowException ex) {
 				// Unrecoverable buffer overflow - close the reader if it was created internally:
-				automaticClose();
+				closeInternallyCreatedReader();
 				throw ex;
-			} catch (IndexOutOfBoundsException ex) {
-				// normal way to catch end of stream.
 			}
 			// streamedText.length() is now guaranteed to return document length
 			// End of stream has been reached, can close the reader if it was created internally:
-			automaticClose();
+			closeInternallyCreatedReader();
 			return null;
 		}
 	}
+	
+	// Ensures internally created reader is closed during garbage collection if the StreamedSource object wasn't closed.
+	// This is held in a separate object because:
+	// - Objects with a finalize method hang around much longer in some GC implementations
+	// - The StreamedSource object very often doesn't even need to be finalized and can contain references to large objects. Obvious case is its use in the CharacterReference.appendDecode method.
+	private static final class InternallyCreatedReaderFinalizer {
+		private Reader reader;
+		
+		public InternallyCreatedReaderFinalizer(Reader reader) {
+			this.reader=reader;
+		}
+
+		public void setReaderToNull() {
+			reader=null;
+		}
+		
+		protected void finalize() {
+			try {
+				if (reader!=null) reader.close();
+			} catch (IOException ex) {}
+		}
+	}
 }
