@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -155,7 +159,7 @@ import java.util.*;
  * </dl>
  */
 public abstract class Tag extends Segment {
-	String name=null; // always lower case, can always use == operator to compare with constants in HTMLElementName interface
+	String name=null; // always lower case, can always use == operator to compare with constants in HTMLElementName interface. Only null in Tag.NOT_CACHED.
 	private Object userData=null;
 	// cached values:
 	Element element=Element.NOT_CACHED;
@@ -438,6 +442,26 @@ public abstract class Tag extends Segment {
 		}
 	}
 
+	// *** consider making public
+	EndTag getNextEndTag() {
+		Tag tag=this;
+		while (true) {
+			tag=tag.getNextTag();
+			if (tag==null) return null;
+			if (tag instanceof EndTag) return (EndTag)tag;
+		}
+	}
+
+	// *** consider making public
+	EndTag getPreviousEndTag() {
+		Tag tag=this;
+		while (true) {
+			tag=tag.getPreviousTag();
+			if (tag==null) return null;
+			if (tag instanceof EndTag) return (EndTag)tag;
+		}
+	}
+
 	// *** consider making public
 	Tag getNextTag(final TagType tagType) {
 		if (tagType==null) return getNextTag();
@@ -521,12 +545,14 @@ public abstract class Tag extends Segment {
 
 	static final Tag getPreviousTag(final Source source, final int pos, final TagType tagType) {
 		// returns null if pos is out of range.
+		if (tagType==null) return getPreviousTag(source,pos);
 		if (source.useSpecialTypesCache) return source.cache.getPreviousTag(pos,tagType);
 		return getPreviousTagUncached(source,pos,tagType,ParseText.NO_BREAK);
 	}
 
 	static final Tag getNextTag(final Source source, final int pos, final TagType tagType) {
 		// returns null if pos is out of range.
+		if (tagType==null) return getNextTag(source,pos);
 		if (source.useSpecialTypesCache) return source.cache.getNextTag(pos,tagType);
 		return getNextTagUncached(source,pos,tagType,ParseText.NO_BREAK);
 	}
