@@ -1,6 +1,6 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 3.2
-// Copyright (C) 2004-2009 Martin Jericho
+// Version 3.4
+// Copyright (C) 2004-2013 Martin Jericho
 // http://jericho.htmlparser.net/
 //
 // This library is free software; you can redistribute it and/or
@@ -14,6 +14,10 @@
 // included in this distribution in the file licence-lgpl-2.1.txt
 // or available at http://www.gnu.org/licenses/lgpl.txt
 //
+// 3. The Apache License version 2.0,
+// included in this distribution in the file licence-apache-2.0.html
+// or available at http://www.apache.org/licenses/LICENSE-2.0.html
+//
 // This library is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
 // See the individual licence texts for more details.
@@ -415,7 +419,7 @@ public abstract class TagType {
 	 * Having this check separated into a different method helps to isolate common code from the code that is unique to each tag type.
 	 * <p>
 	 * A {@linkplain TagType#isServerTag() server tag} is valid in any position except inside a {@linkplain StartTagType#SERVER_COMMON_COMMENT server-side comment},
-	 * but a non-server tag is not valid inside any other tag, nor inside elements with CDATA content such as
+	 * but a non-server tag is not valid inside any other tag, nor inside elements with implicit CDATA content such as
 	 * {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements.
 	 * <p>
 	 * The common implementation of this method behaves differently depending upon whether or not a {@linkplain Source#fullSequentialParse() full sequential parse}
@@ -437,7 +441,7 @@ public abstract class TagType {
 	 * This behaviour is configurable by manipulating the static {@link TagType#getTagTypesIgnoringEnclosedMarkup() TagTypesIgnoringEnclosedMarkup} array
 	 * to determine which tag types can not contain non-server tags in <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
 	 * The {@linkplain TagType#getTagTypesIgnoringEnclosedMarkup() documentation of this property} contains
-	 * a more detailed analysis of the subject and explains why only the {@linkplain StartTagType#COMMENT comment} and 
+	 * a more detailed analysis of the subject, detailing some potential problems with this approach and explaining why only the {@linkplain StartTagType#COMMENT comment} and 
 	 * {@linkplain StartTagType#CDATA_SECTION CDATA section} tag types are included by default.
 	 * <p>
 	 * See the documentation of the <a href="Tag.html#ParsingProcess">tag parsing process</a> for more information about how this method fits into the whole tag parsing process.
@@ -459,19 +463,27 @@ public abstract class TagType {
 	 * If the last tag encountered was the {@linkplain StartTag start tag} of a {@link HTMLElementName#SCRIPT SCRIPT} element,
 	 * the value of this integer is <code>Integer.MAX_VALUE</code>, indicating that no other non-server elements should be recognised until the
 	 * {@linkplain EndTag end tag} of the {@link HTMLElementName#SCRIPT SCRIPT} element is found.
+	 * <p>
+	 * The HTML 4 DTD defines script element content as a special type of CDATA. The XHTML DTD changed it to PCDATA, meaning that HTML elements should be parsed
+	 * inside script elements if they are not escaped by {@linkplain StartTagType#COMMENT comments} or an explicit {@linkplain StartTagType#CDATA_SECTION CDATA section}.
+	 * The <a target="_blank" href="http://dev.w3.org/html5/spec/single-page.html#restrictions-for-contents-of-script-elements">HTML 5 parsing rules</a> reversed this
+	 * again, making it closer to the original HTML 4 rules. Because this parser is designed to facilitate parsing HTML rather than XHTML, it treats script element content
+	 * as implicit CDATA, consistent with HTML 4 and HTML 5.
+	 * <p>
 	 * According to the <a target="_blank" href="http://www.w3.org/TR/html401/types.html#idx-CDATA-1">HTML 4.01 specification section 6.2</a>,
 	 * the first occurrence of the character sequence "<code>&lt;/</code>" terminates the special handling of CDATA within
 	 * {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements.
 	 * This library however only terminates the CDATA handling of {@link HTMLElementName#SCRIPT SCRIPT} element content
-	 * when the character sequence "<code>&lt;/script</code>" is detected, in line with the behaviour of the major browsers.
+	 * when the character sequence "<code>&lt;/script</code>" is detected, in line with the behaviour of the major browsers and with
+	 * <a target="_blank" href="http://dev.w3.org/html5/spec/single-page.html#restrictions-for-contents-of-script-elements">HTML 5 script element parsing rules</a>.
 	 * <p>
-	 * Note that the implicit treatment of {@link HTMLElementName#SCRIPT SCRIPT} element content as CDATA should theoretically also prevent the recognition of
+	 * Note that the implicit treatment of {@link HTMLElementName#SCRIPT SCRIPT} element content as CDATA also prevents the recognition of
 	 * {@linkplain StartTagType#COMMENT comments} and explicit {@linkplain StartTagType#CDATA_SECTION CDATA sections} inside script elements.
-	 * While this is true for explicit {@linkplain StartTagType#CDATA_SECTION CDATA sections}, the parser does still recognise
-	 * {@linkplain StartTagType#COMMENT comments} inside {@link HTMLElementName#SCRIPT SCRIPT} elements in order to maintain compatability with the major browsers.
-	 * This prevents the character sequence "<code>&lt;/script</code>" from terminating the {@link HTMLElementName#SCRIPT SCRIPT} element
-	 * if it occurs inside a {@linkplain StartTagType#COMMENT comment}.  The end of the {@linkplain StartTagType#COMMENT comment} however also
-	 * ends the implicit treatment of the {@link HTMLElementName#SCRIPT SCRIPT} element content as CDATA.
+	 * All major browsers used to recognise comments inside script elements regardless, which is relevant if the script element contains a javascript string literal
+	 * "<code>&lt;script</code>", which would terminate the script element unless it was enclosed in a comment.
+	 * Versions 3.0 to 3.2 of this parser therefore also recognised comments inside script elements in a full sequential parse to maintain compatibility with the
+	 * major browsers, but the latest versions of gecko and webkit browsers now correctly ignore comments inside script elements, so as of version 3.3 this parser
+	 * has also reverted to the correct behaviour.
 	 * <p>
 	 * Although {@link HTMLElementName#STYLE STYLE} elements should theoretically be treated in the same way as {@link HTMLElementName#SCRIPT SCRIPT} elements,
 	 * the syntax of <a target="_blank" href="http://www.w3.org/Style/CSS/">Cascading Style Sheets</a> (CSS) does not contain any constructs that 
@@ -504,14 +516,13 @@ public abstract class TagType {
 			if (fullSequentialParseData[0]==Integer.MAX_VALUE) { // we are in a SCRIPT element
 				if (this==EndTagType.NORMAL && source.getParseText().containsAt("</script",pos)) {
 					// The character sequence "</script" terminates the implicit CDATA section inside the SCRIPT element
+					// Note the tag parsing code will set fullSequentialParseData[0] to the end of the tag if it is parsed successfully. Setting it to pos is just a means to turn off the implicit CDATA section.
 					fullSequentialParseData[0]=pos;
 					return true;
 				}
 				if (this==StartTagType.COMMENT) {
- 					// Although not technically correct, all major browsers also recognise comments inside SCRIPT elements.
- 					// The end of the comment will however terminate the implicit CDATA section inside the SCRIPT element.
-					fullSequentialParseData[0]=pos;
- 					return true;
+ 					// Uncomment the following line if you really want the parser to recognise comments inside script elements, as was the behaviour in v3.0 to v3.2
+ 					//return true;
 				}
 				return false; // reject any other tags inside SCRIPT element
 			}
@@ -519,8 +530,7 @@ public abstract class TagType {
 		}
 		// Use the normal method of checking whether the position is inside a tag of a tag type that ignores enclosed markup:
 		final TagType[] tagTypesIgnoringEnclosedMarkup=getTagTypesIgnoringEnclosedMarkup();
-		for (int i=0; i<tagTypesIgnoringEnclosedMarkup.length; i++) {
-			final TagType tagTypeIgnoringEnclosedMarkup=tagTypesIgnoringEnclosedMarkup[i];
+		for (TagType tagTypeIgnoringEnclosedMarkup : tagTypesIgnoringEnclosedMarkup) {
 			// If this tag type is a comment, don't bother checking whether it is inside another comment.
 			// See javadocs for getTagTypesIgnoringEnclosedMarkup() for more explanation.
 			// Allowing it might result in multiple comments being recognised with the same end delimiter, but the risk of this occuring in a syntactically invalid document
@@ -556,10 +566,15 @@ public abstract class TagType {
 	 * <p>
 	 * A special exception to the algorithm described above applies to {@link StartTagType#COMMENT COMMENT} tags.
 	 * The default implementation of the {@link #isValidPosition(Source,int,int[]) isValidPosition} method
-	 * does not check whether a {@link StartTagType#COMMENT COMMENT} tag is inside another {@link StartTagType#COMMENT COMMENT} tag,
-	 * as this should never happen in a syntactically correct document (the characters '<code>--</code>' should not occur inside a comment).
-	 * Skipping this check also avoids the need to recursively check every {@link StartTagType#COMMENT COMMENT} tag back to the start of the document,
-	 * which has the potential to cause a stack overflow in a large document containing lots of comments.
+	 * does not check whether a {@link StartTagType#COMMENT COMMENT} tag is inside another {@link StartTagType#COMMENT COMMENT} tag.
+	 * The only syntactically valid way that could occur is a construct like <code>&lt;!--<var> ... </var>&lt;!--&gt;</code>, as the characters '<code>--</code>' should not occur inside a comment.
+	 * This construct does actually occur in practice in a 
+	 * {@linkplain MicrosoftConditionalCommentTagTypes#DOWNLEVEL_REVEALED_VALIDATING_IF Microsoft downlevel-revealed conditional comment} tag (<code>&lt;&#33;--[if<var> &#46;&#46;&#46; </var>]&gt;&lt;&#33;--&gt;</code>),
+	 * which may cause problems if documents containing these tags are searched using <a href="Source.html#ParseOnDemand">parse on demand</a> mode.
+	 * The only way to resolve this issue in all instances would be for the default implementation of the {@link #isValidPosition(Source,int,int[]) isValidPosition} method
+	 * to recursively check every {@link StartTagType#COMMENT COMMENT} tag back to the start of the document, which results in far worse problems such as potential stack overflows
+	 * in large documents containing lots of comments and much worse performance in general.
+	 * If the presence of such nested comments is an issue in your case, the best solution is to perform a {@linkplain Source#fullSequentialParse() full sequential parse}.
 	 * <p>
 	 * Theoretically, non-server tags appearing inside any other tag should be ignored, which is how the parser behaves during a
 	 * {@linkplain Source#fullSequentialParse() full sequential parse}.
@@ -674,7 +689,7 @@ public abstract class TagType {
 				final Tag tag=tagType.constructTagAt(source,pos);
 				if (tag!=null) return tag;
 			} catch (IndexOutOfBoundsException ex) {
-				if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(pos).appendTo(new StringBuilder(200).append("Tag at ")).append(" not recognised as type '").append(tagType.getDescription()).append("' because it has no end delimiter").toString());
+				if (source.logger.isErrorEnabled()) source.logger.error(source.getRowColumnVector(pos).appendTo(new StringBuilder(200).append("Tag at ")).append(" not recognised as type '").append(tagType.getDescription()).append("' because it has no end delimiter").toString());
 			}
 		}
 		return null;
