@@ -5,9 +5,11 @@ import net.htmlparser.jericho.*;
 
 @RunWith(Suite.class)
 @Suite.SuiteClasses({
+	TagTest.class,
 	SegmentTest.class,
 	CharacterReferenceTest.class,
 	SegmentGetStyleURISegmentsTest.class,
+	ScriptTest.class,
 	NodeIteratorTest.class,
 	MicrosoftConditionalCommentTagTypesTest.class,
 	StreamedTextTest.class,
