@@ -313,6 +313,12 @@ public class RendererTest {
 	@Test public void testA() throws Exception {
 		assertEquals("My Link <http://mysite.com/>",
 		      render("<a href=\"http://mysite.com/\">My Link</a>"));
+		// don't repeat the URL if it is the same as the text:
+		assertEquals("<http://mysite.com/>",
+		      render("<a href=\"http://mysite.com/\">http://mysite.com/</a>"));
+		// also don't repeat the URL if it is the same as the text, ignoring http:// prefix and / suffix
+		assertEquals("<http://mysite.com/>",
+		      render("<a href=\"http://mysite.com/\">mysite.com</a>"));
 		// line break before URL if it would otherwise be longer than line length:
 		assertEquals("My link with a long label that goes close to the end of the line\n<http://mysite.com/>",
 		      render("<a href=\"http://mysite.com/\">My link with a long label that goes close to the end of the line</a>"));
@@ -377,6 +383,9 @@ public class RendererTest {
 		// decorate <code> with |:
 		assertEquals("|code| text",
 		    renderer("<code>code</code> text").setDecorateFontStyles(true).toString());
+		// make sure decoration appears with text on block boundaries:
+		assertEquals("*bold*\n\n*bold*",
+		    renderer("<p><strong>bold</strong></p><p><strong>bold</strong></p>").setDecorateFontStyles(true).toString());
 	}
 
 	private String render(String sourceText) {
