@@ -59,14 +59,14 @@ public class StreamedSourceTest {
 			segment=i.next();
 			assertEquals(StartTagType.SERVER_COMMON,((Tag)segment).getTagType());
 			assertEquals("<%@ taglib uri=\"/WEB-INF/struts-i18n.tld\" prefix=\"i18n\" %>",segment.toString());
+			assertEquals(120,streamedSource.getBufferSize());
 			segment=i.next();
 			assertEquals("\r\n",segment.toString());
+			assertEquals(240,streamedSource.getBufferSize()); // last next() call fetches the next text segment as well as the following tag, which totals > 120 characters
 			segment=i.next();
 			assertEquals(StartTagType.XML_PROCESSING_INSTRUCTION,((Tag)segment).getTagType());
 			assertEquals("<?xml-stylesheet href=\"standardstyle.css\" title=\"Standard Stylesheet\" type=\"text/css\"?>",segment.toString());
-			assertEquals(120,streamedSource.getBufferSize());
 			segment=i.next();
-			assertEquals(240,streamedSource.getBufferSize()); // last next() call fetches the next text segment as well as the following tag, which totals > 120 characters
 			assertEquals("\r\n",segment.toString());
 			segment=i.next();
 			assertEquals(StartTagType.DOCTYPE_DECLARATION,((Tag)segment).getTagType());
@@ -78,8 +78,10 @@ public class StreamedSourceTest {
 			assertEquals(StartTagType.MARKUP_DECLARATION,((Tag)segment).getTagType());
 			assertEquals("<!ENTITY p CDATA \"<p>\">",segment.toString());
 			for (int x=0; x<7; x++) segment=i.next();
+			assertEquals(240,streamedSource.getBufferSize());
 			assertEquals("Jericho HTML Parser Test Document",segment.toString());
 			for (int x=0; x<5; x++) segment=i.next();
+			assertEquals(480,streamedSource.getBufferSize()); // During the last few calls to next() the 240 character buffer wasn't big enough and was doubled to 480
 			startTag=(StartTag)segment;
 			assertEquals(StartTagType.NORMAL,startTag.getTagType());
 			assertEquals("<meta name=\"keywords\" content=\"HTML parser,test document,R&amp;D\" />",segment.toString());
@@ -144,15 +146,10 @@ public class StreamedSourceTest {
 			segment=i.next();
 			assertEquals("<script language=\"javascript\" type=\"text/javascript\">",segment.toString());
 			segment=i.next();
-			segment=i.next();
-			startTag=(StartTag)segment;
-			assertEquals(StartTagType.COMMENT,startTag.getTagType()); // note that a comment is recognised inside <script> element, whereas anything else isn't.
-			segment=i.next();
+			// note that a comment isn't recognised inside <script> element
 			segment=i.next();
 			assertEquals("</script>",segment.toString());
-			assertEquals(240,streamedSource.getBufferSize());
 			segment=i.next();
-			assertEquals(480,streamedSource.getBufferSize()); // last next() call fetches the next text segment as well as the following comment, which totals > 240 characters
 			segment=i.next();
 			startTag=(StartTag)segment;
 			assertEquals(StartTagType.COMMENT,startTag.getTagType());
@@ -178,13 +175,16 @@ public class StreamedSourceTest {
 			assertEquals(480,streamedSource.getBufferSize());
 
 			// coming up is the very long text segment of 20071 characters
-			for (int x=0; x<41; x++) {
-				// because coalescing is false, it is handled as 41 separate segments of 480 characters (filling the available buffer), plus one extra segment of 391 characters to make up the entire 20071 characters.
+			// because coalescing is false, it is handled in separate chunks with each chunk filling the available 480 character buffer.
+			segment=i.next(); 
+			assertEquals(477,segment.length()); // The first chunk is 477, which is the 480 character buffer minus 3 characters for the preceding <p> tag. The parser can't discard the previous tag because some tags can exist inside other tags.
+			for (int x=0; x<40; x++) {
+				// Next come 40 chunks of the full 480 buffer size.
 				segment=i.next();
 				assertEquals(480,segment.length());
 			}
 			segment=i.next(); 
-			assertEquals(391,segment.length()); // last chunk of the large text
+			assertEquals(394,segment.length()); // last chunk of the large text requires characters to make up the entire 20071 character text.
 			assertEquals(480,streamedSource.getBufferSize()); // buffer hasn't been expanded because we are not coalescing
 
 			segment=i.next();
@@ -227,7 +227,7 @@ public class StreamedSourceTest {
 			// behaviour is identical to non-coalescing until we hit a large non-tag text segment
 			Iterator<Segment> i=streamedSource.iterator();
 			segment=i.next();
-			for (int x=0; x<70; x++) segment=i.next();
+			for (int x=0; x<68; x++) segment=i.next();
 			assertEquals("<p>",segment.toString());
 			assertEquals(480,streamedSource.getBufferSize());
 
@@ -257,7 +257,7 @@ public class StreamedSourceTest {
 	}
 
 	@Test public void testFixedBuffer() throws Exception {
-		char[] buffer=new char[250];
+		char[] buffer=new char[280];
 		StreamedSource streamedSource=null;
 		Segment segment;
 		StartTag startTag;
@@ -272,19 +272,22 @@ public class StreamedSourceTest {
 			for (int x=0; x<14; x++) segment=i.next();
 			segment=i.next();
 			assertEquals("Jericho HTML Parser Test Document",segment.toString());
-			for (int x=0; x<54; x++) segment=i.next();
+			for (int x=0; x<52; x++) segment=i.next();
 			assertEquals("<p>",segment.toString());
 			// coming up is the very long text segment of 20071 characters
-			for (int x=0; x<80; x++) {
-				// because coalescing is false, it is handled as 80 separate segments of 250 characters (filling the available buffer), plus one extra segment of 71 characters to make up the entire 20071 characters.
+			// because coalescing is false, it is handled in chunks of 280 characters (filling the available buffer).
+			segment=i.next(); 
+			assertEquals(277,segment.length()); // The first chunk is 277 characters, which is the buffer size 280 minus the length of the previous <p> tag.
+			for (int x=0; x<70; x++) {
+				// now 70 chunks of full 280 character buffer
 				segment=i.next();
-				assertEquals(250,segment.length());
+				assertEquals(280,segment.length());
 			}
 			segment=i.next(); 
-			assertEquals(71,segment.length()); // last chunk of the large text
+			assertEquals(194,segment.length()); // last chunk of the large text, which is 194 characters to make up the entire 20071 characters.
 			segment=i.next();
 			assertEquals("</p>",segment.toString());
-			assertEquals(250,streamedSource.getBufferSize());
+			assertEquals(280,streamedSource.getBufferSize());
 			try {
 				i.hasNext();
 				// This call to hasNext() attempts to fetch the large comment.
@@ -304,13 +307,13 @@ public class StreamedSourceTest {
 		StartTag startTag;
 		try {
 			streamedSource=new StreamedSource(sourceText);
-			assertEquals(42713,streamedSource.getBufferSize()); // covers entire document
+			assertEquals(42680,streamedSource.getBufferSize()); // covers entire document
 			Iterator<Segment> i=streamedSource.iterator();
 			segment=i.next();
 			for (int x=0; x<11; x++) segment=i.next();
 			assertEquals("<HTML>",segment.toString());
 			StartTag htmlStartTag=(StartTag)segment;
-			for (int x=0; x<59; x++) segment=i.next();
+			for (int x=0; x<57; x++) segment=i.next();
 			assertEquals("<p>",segment.toString());
 			segment=i.next(); // fetches very long text segment
 			assertEquals(20071,segment.toString().length()); // can get text using normal segment.toString()
